Summary: A set of scripts to control installed system facilities
Name: control
Version: 0.8.0
Release: 7%{?dist}
License: GPLv2+
Source0: functions
Source1: control
Source2: control-dump
Source3: control-restore
Source9: control.macros
Source10: control.8
Source11: control-dump.8
Source12: control-restore.8
Source20: ru-control.8
Source21: ru-control-dump.8
Source22: ru-control-restore.8
# shadow-utils
Source100: chage.control
Source101: gpasswd.control
Source102: newgrp.control
Source110: newuidmap.control
Source111: newgidmap.control
# util-linux
Source120: mount.control
Source121: write.control
# pam
Source130: password-hash.control
Source131: password-policy.control
Source140: unix_chkpwd.control
Source141: pam_timestamp_check.control
Requires: sed, grep
BuildArch: noarch
BuildRoot: /override/%name-%version

%define _controldir %_sysconfdir/control.d/facilities

%description
The scripts included in this package provide a common interface to
control system facilities provided by a number of other packages.
This is intended for use primarily by packages which are providing
a facility that can potentially be dangerous to system security,
to let you enable, disable, or configure the facility independently
from package installation.

# These macros are duplicate with what's in control.macros, included here (with
# different names) for use within this spec file itself without requiring this
# package to be installed at its own build time (yet not clashing if it is).
%define _triggerprein_control() \
for facility in %*; do \
	if [ -e %_controldir/"$facility" ]; then \
		/usr/sbin/control-dump "$facility" \
	fi \
done \
%nil

%define _triggerin_control() \
if [ $1 -ge 2 -o $2 -ge 2 ]; then \
	/usr/sbin/control-restore %* \
fi \
%nil

%package shadow-utils
Summary: Control files for shadow-utils
Requires(pre): %name = %version-%release
Requires: %name = %version-%release
Requires: shadow-utils

%description shadow-utils
This package provides control(8) facilities for SUID/SGID/setcap program
binaries coming from the shadow-utils package.

%package util-linux
Summary: Control files for util-linux
Requires(pre): %name = %version-%release
Requires: %name = %version-%release
Requires: util-linux

%description util-linux
This package provides control(8) facilities for SUID/SGID program binaries
coming from the util-linux and util-linux-core packages.

%package pam
Summary: Control files for pam
Requires(pre): %name = %version-%release
Requires: %name = %version-%release
Requires: pam, libxcrypt >= 4.2.0, pam_passwdqc

%description pam
This package provides control(8) facilities for user password hashing scheme
and password policy in use by PAM-aware programs, as well as for SUID helper
program binaries coming from the pam package.

%install
mkdir -p %buildroot{%_controldir,%_sbindir,%_mandir/man8,%_mandir/ru/man8,/var/run/control}
install -p -m755 %_sourcedir/control %buildroot%_sbindir/
install -p -m700 %_sourcedir/control{-dump,-restore} %buildroot%_sbindir/
sed -i s/@VERSION@/%version-%release/ %buildroot%_sbindir/control
install -p -m755 %_sourcedir/functions %buildroot%_sysconfdir/control.d/
for f in control{,-dump,-restore}.8; do
	install -p -m644 %_sourcedir/$f %buildroot%_mandir/man8/
	install -p -m644 %_sourcedir/ru-$f %buildroot%_mandir/ru/man8/$f
done
install -pD -m644 %_sourcedir/control.macros %buildroot%_rpmmacrodir/macros.control

# shadow-utils
install -p -m755 %_sourcedir/chage.control %buildroot%_controldir/chage
install -p -m755 %_sourcedir/gpasswd.control %buildroot%_controldir/gpasswd
install -p -m755 %_sourcedir/newgrp.control %buildroot%_controldir/newgrp
install -p -m755 %_sourcedir/newuidmap.control %buildroot%_controldir/newuidmap
install -p -m755 %_sourcedir/newgidmap.control %buildroot%_controldir/newgidmap

# util-linux
install -p -m755 %_sourcedir/mount.control %buildroot%_controldir/mount
install -p -m755 %_sourcedir/write.control %buildroot%_controldir/write

# pam
install -p -m755 %_sourcedir/password-hash.control %buildroot%_controldir/password-hash
install -p -m755 %_sourcedir/password-policy.control %buildroot%_controldir/password-policy
install -p -m755 %_sourcedir/unix_chkpwd.control %buildroot%_controldir/unix_chkpwd
install -p -m755 %_sourcedir/pam_timestamp_check.control %buildroot%_controldir/pam_timestamp_check

%triggerprein shadow-utils -- shadow-utils
%_triggerprein_control chage gpasswd newgrp newuidmap newgidmap

%triggerin shadow-utils -- shadow-utils
%_triggerin_control chage gpasswd newgrp newuidmap newgidmap

%triggerprein util-linux -- util-linux
%_triggerprein_control mount write

%triggerin util-linux -- util-linux
%_triggerin_control mount write

%triggerprein pam -- pam
%_triggerprein_control password-hash password-policy unix_chkpwd pam_timestamp_check

%triggerin pam -- pam
%_triggerin_control password-hash password-policy unix_chkpwd pam_timestamp_check

%files
%defattr(-,root,root)
%_sbindir/control*
%_rpmmacrodir/macros.control
%dir %_sysconfdir/control.d
%_sysconfdir/control.d/functions
%dir %_controldir
%attr(0700,root,root) %ghost /var/run/control/
%_mandir/man8/*.8*
%_mandir/ru/man8/*.8*

%files shadow-utils
%defattr(-,root,root)
%config %_controldir/chage
%config %_controldir/gpasswd
%config %_controldir/newgrp
%config %_controldir/newuidmap
%config %_controldir/newgidmap

%files util-linux
%defattr(-,root,root)
%config %_controldir/mount
%config %_controldir/write

%files pam
%defattr(-,root,root)
%config %_controldir/password-hash
%config %_controldir/password-policy
%config %_controldir/unix_chkpwd
%config %_controldir/pam_timestamp_check

%changelog
* Wed Jan  3 2024 Solar Designer <solar@openwall.com> 0.8.0-7
- Add unix_chkpwd and pam_timestamp_check facilities to the pam sub-package

* Wed Jan  3 2024 Solar Designer <solar@openwall.com> 0.8.0-6
- Revise password-hash and password-policy to process the underlying two
  "sub-facilities" (for the two configuration files updated by each of these)
  using the same logic that we had used for mount (where the two underlying
  "sub-facilities" are the mount and umount programs)

* Wed Dec 27 2023 Solar Designer <solar@openwall.com> 0.8.0-5
- Install control(8) mode 755 since some of its features work as non-root
- Add sub-package with facilities and triggers for pam password hashing and
  password policy

* Mon Dec 18 2023 Solar Designer <solar@openwall.com> 0.8.0-4
- Add sub-package with facilities and triggers for util-linux

* Mon Dec 18 2023 Solar Designer <solar@openwall.com> 0.8.0-3
- Rename the shadow sub-package to shadow-utils
- Rename the source files not to differentiate them by sub-package
- Add "Requires: shadow-utils" in the shadow-utils sub-package

* Wed Dec 13 2023 Solar Designer <solar@openwall.com> 0.8.0-2
- In addition to Requires(pre), also use Requires in the sub-package
- In %%triggerprein_control, pre-check that the facility exists
- Use (renamed) copies of the trigger macros within this spec file

* Wed Dec 13 2023 Solar Designer <solar@openwall.com> 0.8.0-1
- Add macros for use in RPM triggers
- Add sub-package with facilities and triggers for shadow-utils

* Wed Dec  6 2023 Solar Designer <solar@openwall.com> 0.8.0-0
- Initial packaging for EL based on ALT Linux and Owl packages
